package game;

import sk.thenet.FM;
import sk.thenet.app.*;
import sk.thenet.bmp.*;

class EPlayer extends Entity {
	public var sh:Int = 0; // shape
	public var shpow:Array<Int>; // shape levels
	public var cd:Int = 0; // shot cooldown
	public var cdm:Int = 0; // movecycle cooldown
	public var cdmd:Int = 0; // movecycle direction
	public var paw:Int = 0; // left / right paw
	public var hp:Int = 100; // health
	public var lives:Int = 3;
	public var sheld:Int = 0; // 's' key held
	public var respawn:Int = 0; // respawn timer after death
	public var death:Bool = false; // game over?
	public var hit:Bool = false; // gotten hit?
	
	public var hbx:Int = -10; // hit box
	public var hby:Int = -20;
	public var hbw:Int = 20;
	public var hbh:Int = 40;
	
	public var yfx:Float = 0; // yarnball force
	public var yfy:Float = 0;
	public var ypx:Float = 0;
	public var ypy:Float = 0;
	
	public var dbg_bd:BitmapData;
	
	public function new(ngs:GSGame){
		super(Entity.ET_PLAYER, ngs);
		shpow = [
			1,
			0,
			0,
			0
		];
		x = 120;
		y = 200;
		bdx = 0;
		bdy = 24;
		bdw = 24;
		bdh = 40;
		bdox = -12;
		bdoy = -20;
		dotick = true;
		dbg_bd = new BitmapData(240, 320, 0);
	}
	
	public function collectTreat():Bool {
		if (respawn > 0) return false;
		if (hp < 100){
			hp++;
		}
		gs.scorelvl += 10;
		Noises.playTreat();
		return true;
	}
	
	public function collectLevel(lsh:Int):Bool {
		if (respawn > 0) return false;
		if (shpow[lsh] < 3){
			shpow[lsh]++;
		}
		gs.scorelvl += 200;
		Noises.playLevel();
		return true;
	}
	
	public function collectLife():Bool {
		if (respawn > 0) return false;
		if (lives < 10){
			lives++;
		}
		Noises.playLife();
		return true;
	}
	
	public function damage(dmg:Int):Void {
		if (respawn > 0) return;
		hurt = 5;
		hp -= dmg;
		hit = true;
		if (hp <= 0){
			hp = 0;
			gs.spawnExplosion(x, y, 75);
			Noises.playDeath();
			if (lives == 0){
				respawn = 300;
				death = true;
			} else {
				lives--;
				respawn = 150;
				hp = 100;
				x = 120;
				y = 200;
				sh = 0;
				cd = 0;
				cdm = 0;
				hurt = 10;
				var ldown:Array<Int> = [];
				if (shpow[0] > 1) ldown.push(0);
				if (shpow[1] > 0) ldown.push(1);
				if (shpow[2] > 0) ldown.push(2);
				if (shpow[3] > 0) ldown.push(3);
				shpow[ldown[Main.prng.nextMod(ldown.length)]]--;
			}
		} else {
			Noises.playHurt();
		}
	}
	
	override public function render(bd:BitmapData):Void {
		if (sh == 1 && cd >= 4){
			bd.copyPixels(Assets.aGame, x - 20, y - 36, 48 + paw * 40, 64 + ((10 - cd) >> 1) * 24, 40, 24);
		}
	}
	
	public function tileCollide(vx:Int, vy:Int):Void {
		//dbg_bd.fill(0);
		//dbg_bd.fillRect(x + hbx, y + hby, hbw, hbh, 0xFFFF0000);
		
		y++;
		vy--;
		
		ypx += yfx;
		ypy += yfy;
		yfx = 0;
		yfy = 0;
		while (ypx > 0){
			ypx--;
			vx++;
		}
		while (ypx < 0){
			ypx++;
			vx--;
		}
		while (ypy > 0){
			ypy--;
			vy++;
		}
		while (ypy < 0){
			ypy++;
			vy--;
		}
		
		var lphy:Int = gs.lvl.ph >> 3;
		var lphsy:Int = gs.lvl.ph % 8;
		var ocol = false;
		
		while (vx != 0 || vy != 0){
			var svx:Int = 0;
			var svy:Int = 0;
			if (FM.absI(vx) >= FM.absI(vy)){
				if (vx > 0){
					vx--;
					svx = 1;
				} else {
					vx++;
					svx = -1;
				}
			} else {
				if (vy > 0){
					vy--;
					svy = 1;
				} else {
					vy++;
					svy = -1;
				}
			}
			if (svx != 0){
				if (x + hbx == 0 && svx < 0){
					continue;
				} else if (x + hbx + hbw == 240 && svx > 0){
					continue;
				}
				var col = false;
				if ((x + hbx + (svx < 0 ? 0 : hbw)) % 8 == 0){
					var tx:Int = (x + hbx + (svx < 0 ? -1 : hbw + 1)) >> 3;
					var ty:Int = 40 - ((y - lphsy + hby) >> 3);
					for (sy in 0...6){
						var lvli:Int = tx + (ty - sy + lphy) * 30;
						var lvlt:Int = (lvli >= 0 && lvli < gs.lvl.tdata.length ? gs.lvl.tdata[lvli] : 0);
						if (lvlt == 1 || lvlt == 10){
							//dbg_bd.fillRect(tx << 3, 320 - ((ty - sy) << 3) + lphsy, 8, 8, 0xFF00AA00);
							col = true;
						} else {
							//dbg_bd.fillRect(tx << 3, 320 - ((ty - sy) << 3) + lphsy, 8, 8, 0xFF0000AA);
						}
					}
				}
				if (!col){
					x += svx;
				} else {
					ocol = true;
				}
			} else {
				if (y + hby == 0 && svy < 0){
					continue;
				} else if (y + hby + hbh == 320 && svy > 0){
					continue;
				}
				var col = false;
				var tx:Int = (x + hbx) >> 3;
				var ty:Int = 40 - ((y - lphsy + hby + (svy < 0 ? -1 : hbh - 2)) >> 3);
				for (sx in 0...3 + ((x + hbx) % 8 < 5 ? 0 : 1)){
					var lvli:Int = (tx + sx) + (ty + lphy) * 30;
					var lvlt:Int = (lvli >= 0 && lvli < gs.lvl.tdata.length ? gs.lvl.tdata[lvli] : 0);
					if (lvlt == 1 || lvlt == 10){
						//dbg_bd.fillRect((tx + sx) << 3, 320 - (ty << 3) + lphsy, 8, 8, 0xFF00AA00);
						col = true;
					} else {
						//dbg_bd.fillRect((tx + sx) << 3, 320 - (ty << 3) + lphsy, 8, 8, 0xFF0000AA);
					}
				}
				if (!col){
					y += svy;
				} else {
					ocol = true;
				}
			}
		}
		if (ocol && !death && respawn == 0){
			Noises.playCollide();
			damage(3);
		}
		if (x + hbx < 0) x = -hbx;
		if (x + hbx + hbw > 240) x = 240 - hbx - hbw;
		if (y + hby < 0) y = -hby;
		if (y + hby + hbh > 320) y = 320 - hby - hbh;
	}
	
	override public function tick(st:SApp):Void {
		var vx:Int = 0;
		var vy:Int = 0;
		
		if (death){
			visible = false;
			if (respawn > 0){
				respawn--;
			} else {
				Main.inst.applyState(Main.inst.getStateById("gameover"));
			}
			return;
		}
		if (respawn > 0){
			if (respawn <= 60){
				visible = respawn % 2 == 0;
				if (!st.kb.down[Keyboard.K_S] && st.kb.down[Keyboard.K_LEFT] != st.kb.down[Keyboard.K_RIGHT]){
					vx = (st.kb.down[Keyboard.K_LEFT] ? -3 : 3);
				}
				if (!st.kb.down[Keyboard.K_S] && st.kb.down[Keyboard.K_UP] != st.kb.down[Keyboard.K_DOWN]){
					vy = (st.kb.down[Keyboard.K_UP] ? -3 : 3);
				}
				tileCollide(vx, vy);
			} else {
				visible = false;
			}
			respawn--;
			return;
		}
		visible = true;
		var ce = gs.entsep[Entity.ET_ENEMY].n;
		while (ce != null && ce.type == Entity.ET_ENEMY){
			if (x + hbx + hbw >= ce.x + ce.bdox && x + hbx < ce.x + ce.bdox + ce.bdw
				&& y + hby + hbh >= ce.y + ce.bdoy && y + hby < ce.y + ce.bdoy + ce.bdh){
				(cast ce:EEnemy).hp -= 1;
				ce.hurt = 2;
				Noises.playCollide();
				damage(3);
			}
			ce = ce.n;
		}
		var sp:Int = (sh == 1 ? shpow[1] : 3);
		if (!gs.lvlover && !st.kb.down[Keyboard.K_S] && st.kb.down[Keyboard.K_LEFT] != st.kb.down[Keyboard.K_RIGHT]){
			vx = (st.kb.down[Keyboard.K_LEFT] ? -sp : sp);
		}
		if (!gs.lvlover && !st.kb.down[Keyboard.K_S] && st.kb.down[Keyboard.K_UP] != st.kb.down[Keyboard.K_DOWN]){
			vy = (st.kb.down[Keyboard.K_UP] ? -sp : sp);
		}
		if (cd > 0){
			cd--;
		}
		if (!gs.lvlover && (st.kb.down[Keyboard.K_SPACE] || st.kb.down[Keyboard.K_A]) && cd == 0 && cdm == 0){
			if (sh == 0){
				cd = switch (shpow[0]){
					case 1: 10;
					case _: 8;
				};
				var nbul = new EBullet(false, x + 4, y - 20, gs);
				nbul.bdmg = switch (shpow[0]){
					case 1: 5;
					case 2: 10;
					case _: 20;
				};
				gs.addEntity(nbul);
				Noises.playHitPlayer();
			} else if (sh == 1){
				cd = switch (shpow[1]){
					case 1: 10;
					case _: 9;
				};
				paw++;
				paw %= 2;
				Noises.playHitPlayer();
			} else if (sh == 2){
				cd = 10 - shpow[2];
				switch (shpow[2]){
					case 1:
					gs.addEntity(new EBullet(false, x + 4 - (paw * 12), y - 20, gs, EBullet.BT_PLAYER_NINJA2));
					gs.addEntity(new EBullet(false, x + 4 - (paw * 12), y - 20, gs, EBullet.BT_PLAYER_NINJA3));
					gs.addEntity(new EBullet(false, x + 4 - (paw * 12), y - 20, gs, EBullet.BT_PLAYER_NINJA4));
					case 2:
					if (paw % 2 == 0) gs.addEntity(new EBullet(false, x + 4 - (paw * 12), y - 20, gs, EBullet.BT_PLAYER_NINJA1));
					gs.addEntity(new EBullet(false, x + 4 - (paw * 12), y - 20, gs, EBullet.BT_PLAYER_NINJA2));
					gs.addEntity(new EBullet(false, x + 4 - (paw * 12), y - 20, gs, EBullet.BT_PLAYER_NINJA3));
					gs.addEntity(new EBullet(false, x + 4 - (paw * 12), y - 20, gs, EBullet.BT_PLAYER_NINJA4));
					if (paw % 2 == 1) gs.addEntity(new EBullet(false, x + 4 - (paw * 12), y - 20, gs, EBullet.BT_PLAYER_NINJA5));
					case 3:
					gs.addEntity(new EBullet(false, x + 4 - (paw * 12), y - 20, gs, EBullet.BT_PLAYER_NINJA1));
					gs.addEntity(new EBullet(false, x + 4 - (paw * 12), y - 20, gs, EBullet.BT_PLAYER_NINJA2));
					gs.addEntity(new EBullet(false, x + 4 - (paw * 12), y - 20, gs, EBullet.BT_PLAYER_NINJA3));
					gs.addEntity(new EBullet(false, x + 4 - (paw * 12), y - 20, gs, EBullet.BT_PLAYER_NINJA4));
					gs.addEntity(new EBullet(false, x + 4 - (paw * 12), y - 20, gs, EBullet.BT_PLAYER_NINJA5));
				}
				paw++;
				paw %= 2;
				Noises.playHitPlayer();
			}
		} else if (!gs.lvlover && st.kb.down[Keyboard.K_S] && cd == 0 && cdm == 0){
			if (st.kb.down[Keyboard.K_LEFT]){
				sh = 0;
			} else if (st.kb.down[Keyboard.K_RIGHT] && shpow[1] != 0){
				sh = 1;
			} else if (st.kb.down[Keyboard.K_UP] && shpow[2] != 0){
				sh = 2;
			} else if (st.kb.down[Keyboard.K_DOWN] && shpow[3] != 0){
				sh = 3;
			}
			if (sheld < 20){
				sheld++;
			}
		}
		if (!st.kb.down[Keyboard.K_S]){
			sheld = 0;
		}
		if (sh == 0){
			bdx = (cd > 5 ? 24 : 0);
			bdy = 24;
			bdh = 40;
			bddyn = false;
		} else if (sh == 1){
			bdx = (cd > 5 ? 24 : 0);
			bdy = 64;
			bdh = 32;
			if (cd >= 4){
				bddyn = true;
				var ce = gs.entsep[Entity.ET_ENEMY].n;
				var col = false;
				while (ce != null && ce.type == Entity.ET_ENEMY){
					if (x + 20 >= ce.x + ce.bdox && x - 20 < ce.x + ce.bdox + ce.bdw
						&& y - 12 >= ce.y + ce.bdoy && y - 36 < ce.y + ce.bdoy + ce.bdh){
						(cast ce:EEnemy).hp -= switch (shpow[1]){
							case 1: 5;
							case 2: 10;
							case _: 15;
						};
						ce.hurt = 2;
						col = true;
						break;
					}
					ce = ce.n;
				}
			} else {
				bddyn = false;
			}
		} else {
			bdh = 48;
			if (cd >= 2){
				bdx = paw * 24;
				bdy = 192;
			} else {
				bdx = 0;
				bdy = 96;
			}
			if (vx != 0){
				if (cd != 0){
					vx = 0;
				} else if (cdm == 0){
					cdm = 16;
					cdmd = (vx > 0 ? 0 : 1);
				}
			}
			if (cdm != 0){
				if (cdmd == 0){
					if (cdm >= 13){
						bdx = 0;
					} else if (cdm >= 9){
						bdx = 24;
					} else if (cdm >= 5){
						bdx = 0;
						bdy += 48;
					} else if (cdm >= 1){
						bdx = 24;
						bdy += 48;
					}
				} else {
					if (cdm >= 13){
						bdx = 0;
					} else if (cdm >= 9){
						bdx = 24;
						bdy += 48;
					} else if (cdm >= 5){
						bdx = 0;
						bdy += 48;
					} else if (cdm >= 1){
						bdx = 24;
					}
				}
				cdm--;
			}
		}
		tileCollide(vx, vy);
	}
}