package sk.thenet.app;

import sk.thenet.event.*;
import sk.thenet.bmp.*;
import sk.thenet.logic.*;

class SApp implements Stater<SSApp, STApp> {
	public var currentState:SSApp;
	public var states:Array<SSApp>;
	public var transitions:Array<STApp>;
	public var bd:BitmapData;
	public var scale:Int = 1;
	public var es:ESApp;
	public var kb:Keyboard;
	
	public function new(nstate:SSApp = null){
		if (nstate == null){
			nstate = new SSApp("uninit", this);
		}
		currentState = nstate;
		states = [
			nstate
		];
		transitions = [];
		es = new ESApp();
		es.listen("tick", tick);
	}
	
	public function initBitmap(w:Int, h:Int, nscale:Int):Void {
		scale = nscale;
		bd = new BitmapData(w, h, 0);
		BitmapSurface.createMainSurface(bd, scale);
	}
	
	public function initMouse():Void {
		es.listen("mclick", mClick);
		es.listen("mdown", mDown);
		es.listen("mup", mUp);
	}
	
	public function initKeyboard():Void {
		kb = new Keyboard(this);
		//es.listen("kdown", kDown);
		//es.listen("kup", kUp);
	}
	
	public function getStateById(id:String):SSApp {
		for (s in states){
			if (s.id == id) return s;
		}
		return null;
	}
	
	public inline function getTransition(id:String):STApp {
		for (s in transitions){
			if (s.id == id) return s;
		}
		return null;
	}
	
	public inline function tick(e:ETick):Bool {
		currentState.tick();
		return true;
	}
	
	public inline function mClick(e:EMEvent.EMClick):Bool {
		currentState.mClick(e.x >> scale, e.y >> scale);
		return true;
	}
	
	public inline function mDown(e:EMEvent.EMDown):Bool {
		currentState.mDown(e.x >> scale, e.y >> scale);
		return true;
	}
	
	public inline function mUp(e:EMEvent.EMUp):Bool {
		currentState.mUp(e.x >> scale, e.y >> scale);
		return true;
	}
	
	public inline function kDown(e:EKEvent.EKDown):Bool {
		currentState.kDown(e.kcode);
		return true;
	}
	
	public inline function kUp(e:EKEvent.EKUp):Bool {
		currentState.kUp(e.kcode);
		return true;
	}
	
	public inline function addState(nstate:SSApp):Void {
		states.push(nstate);
	}
	
	public inline function addTransition(ntransition:STApp):Void {
		transitions.push(ntransition);
	}
	
	public inline function applyTransition(transition:STApp):SSApp {
		transition.apply(this);
		return currentState;
	}
	
	public inline function applyState(nstate:SSApp):Void {
		if (currentState == null || currentState.from()){
			currentState = nstate;
			currentState.to();
		}
	}
	
	public inline function getState():SSApp {
		return currentState;
	}
}