package sk.thenet.bmp;

private typedef UIVector = #if flash10
	flash.Vector<UInt>;
#else
	haxe.ds.Vector<UInt>;
#end

private typedef RBmpData = #if flash10
	flash.display.BitmapData;
#else
	{data:UIVector, width:Int, height: Int};
#end

abstract BitmapData(RBmpData){
	public var width(get, never):Int;
	inline function get_width():Int return this.width;
	public var height(get, never):Int;
	inline function get_height():Int return this.height;
	
	/**
	nw		width
	nh		height
	nfc		fill colour
	**/
	public inline function new(nw:Int, nh:Int, nfc:UInt){
		#if flash10
			this = new RBmpData(nw, nh, true, nfc);
		#else
			this = {data: new UIVector(nw * nh), width: nw, height: nh};
			for (i in 0...this.data.length) this.data[i] = nfc;
		#end
	}
	
	public inline function get(x:Int, y:Int):UInt {
		#if flash10
			return this.getPixel32(x, y);
		#else
			if (x < 0 || y < 0 || x >= this.width || y >= this.height) return 0;
			return this.data[x + y * this.width];
		#end
	}
	
	public inline function set(x:Int, y:Int, c:UInt):Void {
		#if flash10
			this.setPixel32(x, y, c);
		#else
			if (x < 0 || y < 0 || x >= this.width || y >= this.height) return;
			this.data[x + y * width] = c;
		#end
	}
	
	public inline function setAlpha(x:Int, y:Int, c:UInt, aa:Float = 1.0):Void {
		if (x < 0 || y < 0 || x >= this.width || y >= this.height) return;
		#if flash10
			var orig:UInt = this.getPixel32(x, y);
		#else
			var orig:UInt = this.data[x + y * this.width];
		#end
		var dsta:Float = (((orig >> 24) & 0xFF) / 255);
		/*
		if (dsta == 0){
			#if flash10
				this.setPixel32(x, y, c);
			#else
				this.data[x + y * this.width] = c;
			#end
		} else {*/
			var srca:Float = (((c >> 24) & 0xFF) / 255) * aa;
			var srcasub:Float = 1 - srca;
			var outa:Float = srca + dsta * srcasub;
			//if (outa > 1) outa = 1;
			#if flash10
				this.setPixel32(x, y,
			#else
				this.data[x + y * this.width] =
			#end
					(FM.floor(outa * 255) << 24)
					| (FM.floor((((c >> 16) & 0xFF) * srca + ((orig >> 16) & 0xFF) * dsta * srcasub) / outa) << 16)
					| (FM.floor((((c >> 8) & 0xFF) * srca + ((orig >> 8) & 0xFF) * dsta * srcasub) / outa) << 8)
					| FM.floor(((c & 0xFF) * srca + (orig & 0xFF) * dsta * srcasub) / outa)
			#if flash10
				)
			#end
			;
		//}
	}
	
	public inline function getVector(x:Int, y:Int, w:Int, h:Int):UIVector {
		x = (x < 0 ? 0 : (x >= this.width ? this.width - 1 : x));
		y = (y < 0 ? 0 : (y >= this.height ? this.height - 1 : y));
		w = (w < 1 ? 1 : (w > (this.width - x) ? this.width - x : w));
		h = (h < 1 ? 1 : (h > (this.height - y) ? this.height - y : h));
		#if flash10
			return this.getVector(new flash.geom.Rectangle(x, y, w, h));
		#else
			if (x == 0 && y == 0 && w == this.width && h == this.height) return this.data;
			var ret:UIVector = new UIVector((w - x) * (h - y));
			var ais:UInt = 0;
			var ait:UInt = 0;
			for (ay in y...(y + h)){
				ais = x + ay * this.width;
				for (ax in x...(x + w)){
					ret[ait] = this.data[ais];
					ais++;
					ait++;
				}
			}
			return ret;
		#end
	}
	
	public inline function getFullVector():UIVector {
		#if flash10
			return this.getVector(this.rect);
		#else
			return this.data;
		#end
	}
	
	public inline function setVector(v:UIVector, x:Int, y:Int, w:Int, h:Int):Void {
		x = (x < 0 ? 0 : (x >= this.width ? this.width - 1 : x));
		y = (y < 0 ? 0 : (y >= this.height ? this.height - 1 : y));
		w = (w < 1 ? 1 : (w > (this.width - x) ? this.width - x : w));
		h = (h < 1 ? 1 : (h > (this.height - y) ? this.height - y : h));
		#if flash10
			this.setVector(new flash.geom.Rectangle(x, y, w, h), v);
		#else
			if (x == 0 && y == 0 && w == this.width && h == this.height) this.data = v;
			var ais:UInt = 0;
			var ait:UInt = 0;
			for (ay in y...(y + h)){
				ais = x + ay * this.width;
				for (ax in x...(x + w)){
					this.data[ais] = v[ait];
					ais++;
					ait++;
				}
			}
		#end
	}
	
	public inline function setFullVector(v:UIVector):Void {
		#if flash10
			this.setVector(cast(this, flash.display.BitmapData).rect, v);
		#else
			this.data = v;
		#end
	}
	
	public inline function copyPixels(src:BitmapData, tx:Int, ty:Int, sx:Int, sy:Int, sw:Int, sh:Int):Void {
		#if flash10
			this.copyPixels(cast(src, flash.display.BitmapData), new flash.geom.Rectangle(sx, sy, sw, sh),
				new flash.geom.Point(tx, ty), null, null, true);
		#end
	}
	
	public inline function copyPixelsMasked(src:BitmapData, tx:Int, ty:Int, sx:Int, sy:Int, sw:Int, sh:Int,
		m:BitmapData, mx:Int, my:Int):Void {
		#if flash10
			this.copyPixels(cast(src, flash.display.BitmapData), new flash.geom.Rectangle(sx, sy, sw, sh),
				new flash.geom.Point(tx, ty), cast(m, flash.display.BitmapData), new flash.geom.Point(mx, my), true);
		#end
	}
	
	public inline function threshold(src:BitmapData, tx:Int, ty:Int, sx:Int, sy:Int, sw:Int, sh:Int, op:String, thr:UInt,
		c:UInt = 0, m:UInt = 0xFFFFFFFF, cs:Bool = false):UInt {
		#if flash10
			return this.threshold(cast(src, flash.display.BitmapData), new flash.geom.Rectangle(sx, sy, sw, sh),
				new flash.geom.Point(tx, ty), op, thr, c, m, cs);
		#else
			return 0;
		#end
	}
	
	public inline function thresholdSelf(op:String, thr:UInt, c:UInt = 0, m:UInt = 0xFFFFFFFF, cs:Bool = false):UInt {
		#if flash10
			return this.threshold(cast(this, flash.display.BitmapData), cast(this, flash.display.BitmapData).rect,
				new flash.geom.Point(0, 0), op, thr, c, m, cs);
		#else
			return 0;
		#end
	}
	
	public inline function lock():Void {
		#if flash10
			this.lock();
		#end
	}
	
	public inline function unlock():Void {
		#if flash10
			this.unlock();
		#end
	}
	
	public inline function fill(c:UInt):Void {
		#if flash10
			this.fillRect(this.rect, c);
		#end
	}
	
	public inline function fillRect(x:Int, y:Int, w:Int, h:Int, c:UInt):Void {
		#if flash10
			this.fillRect(new flash.geom.Rectangle(x, y, w, h), c);
		#end
	}
	
	public inline function cut(x:Int, y:Int, w:Int, h:Int):BitmapData {
		#if flash10
			var ret:BitmapData = new BitmapData(w, h, 0);
			ret.copyPixels(cast(this, BitmapData), 0, 0, x, y, w, h);
			return ret;
		#else
			return null;
		#end
	}
	
	public inline function colourOut(c:UInt):BitmapData {
		#if flash10
			var ret = new BitmapData(width, height, 0);
			var ov = getFullVector();
			var bdv = ret.getFullVector();
			for (i in 0...bdv.length){
				if ((ov[i] & 0xFF000000) != 0) bdv[i] = c;
			}
			ret.setFullVector(bdv);
			return ret;
		#else
			return null;
		#end
	}
}