package sk.thenet.bmp;

import sk.thenet.FM;
import haxe.ds.Vector;
import haxe.io.Bytes;

class BitmapFont {
	public static function makeMonospaced(nbd:BitmapData, bdw:Int, nbdo:Int, nchw:Int, nchh:Int, nchwd:Int = 0):BitmapFont {
		var ret = new BitmapFont();
		ret.bdo = nbdo;
		ret.chh = nchh;
		var bdh:Int = FM.floor(nbd.height / nchh);
		ret.chw = new Vector<Int>(bdw * bdh);
		ret.bds = new Vector<BitmapData>(bdw * bdh);
		var i:Int = 0;
		for (y in 0...bdh){
			for (x in 0...bdw){
				ret.chw[i] = nchw + nchwd;
				ret.bds[i] = nbd.cut(x * nchw, y * nchh, nchw, nchh);
				i++;
			}
		}
		return ret;
	}
	
	public var bdo:Int; // bd charcode offset
	public var bds:Vector<BitmapData>; // glyphs
	public var chw:Vector<Int>; // char width
	public var chh:Int; // char height
	
	public function new(){}
	
	public function recolour(c:UInt):BitmapFont {
		var ret = new BitmapFont();
		ret.bdo = bdo;
		ret.bds = new Vector<BitmapData>(bds.length);
		ret.chw = new Vector<Int>(chw.length);
		ret.chh = chh;
		c = c & 0xFFFFFF;
		for (i in 0...bds.length){
			ret.bds[i] = new BitmapData(bds[i].width, bds[i].height, 0);
			var v = bds[i].getFullVector();
			var nv = ret.bds[i].getFullVector();
			for (j in 0...v.length){
				nv[j] = (v[j] & 0xFF000000) | c;
			}
			ret.bds[i].setFullVector(nv);
			ret.chw[i] = chw[i];
		}
		return ret;
	}
	
	public function render(tbd:BitmapData, txt:String, sx:Int, sy:Int, mw:Int):Void {
		var txtbyt = Bytes.ofString(txt).getData();
		var x:Int = sx;
		var xc:Int = 0;
		var y:Int = sy;
		var ch:Int = 0;
		for (i in 0...txtbyt.length){
			ch = Bytes.fastGet(txtbyt, i);
			if (ch == 0x0A){
				x = sx;
				xc = 0;
				y += chh;
				continue;
			}
			ch -= bdo;
			if (xc < mw && ch >= 0 && ch < bds.length){
				tbd.copyPixels(bds[ch], x, y, 0, 0, bds[ch].width, chh);
				x += chw[ch];
			}
			xc++;
		}
	}
}