package sk.thenet.plustd.audio.music;

import haxe.ds.Vector;
import sk.thenet.FM;

class MMPatternBasic extends MMPattern {
	public static function fromNotes(pat:String, ntempo:Float):MMPatternBasic {
		var notes:Array<String> = pat.split(" ");
		var lnote:Int = -1;
		var nnotes:Array<Float> = [];
		var nfreq:Array<Float> = [];
		var ntrig:Array<Int> = [];
		var effects:Array<MMEffect> = [];
		var effects_step:Array<Int> = [];
		for (nf in 0...notes.length){
			var nfe = notes[nf].split(";");
			var n = nfe[0];
			if (n == "---"){
				nnotes.push(lnote);
				nfreq.push(MusicManager.indexToFreq(lnote));
				ntrig.push(0);
			} else if (n == "STP"){
				nnotes.push(lnote);
				nfreq.push(MusicManager.indexToFreq(lnote));
				ntrig.push(-1);
			} else if (n == "TRG"){
				nnotes.push(-1);
				nfreq.push(-1);
				ntrig.push(1);
			} else if (n.charAt(0) == "+"){
				lnote = lnote + Std.parseInt(n.substr(1));
				nnotes.push(lnote);
				nfreq.push(MusicManager.indexToFreq(lnote));
				ntrig.push(1);
			} else {
				lnote = MusicManager.NOTES.indexOf(n);
				nnotes.push(lnote);
				nfreq.push(MusicManager.noteToFreq(n, 0));
				ntrig.push(1);
			}
			for (ni in 1...nfe.length){
				var par:Array<String> = nfe[ni].substr(3).split(",");
				switch (nfe[ni].substr(0, 3)){
					case "sli":
					effects.push(new MMESlide(false, Std.parseInt(par[0]),
						MMPattern.countSamples(Std.parseFloat(par[1]), ntempo)));
					case "arp":
					effects.push(new MMEArpeggio(Std.parseInt(par[0]), Std.parseInt(par[1]), Std.parseInt(par[2]),
						MMPattern.countSamples(Std.parseFloat(par[3]), ntempo)));
					case "slj":
					effects.push(new MMESlide(true, Std.parseInt(par[0]),
						MMPattern.countSamples(Std.parseFloat(par[1]), ntempo)));
					case "VOL":
					effects.push(new MMEFade(true, MMPattern.countSamples(Std.parseFloat(par[0]), ntempo)));
					case "vol":
					effects.push(new MMEFade(false, MMPattern.countSamples(Std.parseFloat(par[0]), ntempo)));
					case "=te":
					effects.push(new MMESetter(MMESetter.SP_TEMPO, Std.parseFloat(par[0])));
					default: continue;
				}
				effects_step.push(nf);
			}
		}
		var ret = new MMPatternBasic(nnotes, nfreq, ntrig, ntempo);
		ret.effects = effects;
		ret.effects_step = effects_step;
		for (e in ret.effects){
			if (e.effid == 1){
				cast(e, MMESetter).pat = ret;
			}
		}
		return ret;
	}
	public static function fromFreq(nfreq:Array<Float>, ntempo:Float):MMPatternBasic {
		var ln2:Float = Math.log(2);
		var lnote:Float = 0;
		return new MMPatternBasic([
			for (n in nfreq){
				if (n < 0) lnote;
				else {
					lnote = 12 * (Math.log(n / 440) / ln2) + 45;
				}
			}
		], [
			for (n in nfreq){
				if (n < 0) lnote;
				else {
					lnote = n;
					MusicManager.SAMPLERATE / n;
				}
			}
		], [
			for (n in nfreq){
				if (n < 0) 0;
				else 1;
			}
		], ntempo);
	}
	
	public var notes:Array<Float>;
	public var freq:Array<Float>;
	public var trig:Array<Int>;
	public var fph:Int = 0;
	public var tph:Float = 0;
	public var fph_trig:Bool = false;
	
	public function new(nnotes:Array<Float>, nfreq:Array<Float>, ntrig:Array<Int>, ntempo:Float){
		super(ntempo);
		notes = nnotes;
		freq = nfreq;
		trig = ntrig;
	}
	
	override public function reset():Void {
		super.reset();
		fph = 0;
		tph = 0;
		fph_trig = false;
	}
	
	override public inline function setFreq(instr:MMInstrument, c:Int):Void {
		if (instr != null){
			instr.notes = notes[fph];
			instr.freq = freq[fph];
			instr.trig = (fph_trig ? 0 : trig[fph]);
		}
		fph_trig = true;
		if (!eftrig && effects.length > 0 && effects_step[efph] == fph){
			var lg:Int = 0;
			while (lg++ < effects.length && effects_step[efph] == fph){
				if (instr != null) instr.addPatternEffect(effects[efph], c);
				effects[efph].trigger(1, MMEffect.ET_PATTERN);
				efph++;
				efph %= effects.length;
			}
			eftrig = true;
		}
		tph += 1;
		if (tph >= tempo){
			fph++;
			fph_trig = false;
			eftrig = false;
			while (fph >= freq.length){
				for (e in effects){
					e.reset();
				}
				fph -= freq.length;
			}
			tph -= tempo;
		}
	}
}