package sk.thenet.prog;

import haxe.ds.Vector;

class GeneratorModifier<T> {
	public var gen:Iterator<T>;
	public var icur:UInt = 0;
	public var istart:UInt = 0;
	public var iendlimit:Bool = false;
	public var iend:UInt = 0;
	public var skipmode:UInt = 0;
	public var skipi:UInt = 0;
	public var skipmax:UInt = 0;
	public var skipmax2:UInt = 0;
	public var skipseq:Vector<Bool>;
	
	public function new(ngen:Iterator<T>){
		gen = ngen;
	}
	
	public inline function modifyRange(nstart:UInt, nend:UInt):GeneratorModifier<T> {
		if (nstart > nend){
			var t:UInt = nstart;
			nstart = nend;
			nend = t;
		}
		istart = nstart;
		iendlimit = true;
		iend = nend;
		return this;
	}
	
	public inline function modifyCount(ncnt:UInt):GeneratorModifier<T> {
		modifyRange(istart, istart + ncnt);
		return this;
	}
	
	public function skip(nskip:UInt):GeneratorModifier<T> {
		skipmode = 1;
		skipmax2 = nskip;
		skipmax = skipmax2 << 1;
		return this;
	}
	
	public function skipSequence(nseq:Array<Int>):GeneratorModifier<T> {
		skipmode = 2;
		var len:Int = 0;
		for (sp in nseq){
			len += sp;
		}
		skipmax = len;
		skipseq = new Vector<Bool>(len);
		var i:UInt = 0;
		var on:Bool = true;
		for (sp in nseq){
			for (j in 0...sp){
				skipseq[i] = on;
				i++;
			}
			on = !on;
		}
		return this;
	}
	
	private inline function fastForward():Void {
		if (icur < istart){
			while (gen.hasNext() && icur < istart){
				gen.next();
				icur++;
			}
		}
		if (skipmode == 1){
			while (gen.hasNext() && skipi % skipmax >= skipmax2 && (!iendlimit || icur < iend)){
				gen.next();
				icur++;
				skipi++;
				skipi %= skipmax;
			}
		} else if (skipmode == 2){
			while (gen.hasNext() && !skipseq[skipi] && (!iendlimit || icur < iend)){
				gen.next();
				icur++;
				skipi++;
				skipi %= skipmax;
			}
		}
	}
	
	public function hasNext():Bool {
		fastForward();
		if (iendlimit && icur >= iend) return false;
		return gen.hasNext();
	}
	
	public function next():T {
		fastForward();
		if (!gen.hasNext()) return null;
		icur++;
		if (skipmode != 0){
			skipi++;
			skipi %= skipmax;
		}
		return gen.next();
	}
}