package sk.thenet.prog;

import sk.thenet.model.Point2D;
import sk.thenet.FM;
import sk.thenet.bmp.*;

class LGBresenham implements LineGenerator<Int> {
	public var lens:Int;
	public var lenl:Int;
	public var ylong:Bool = false;
	public var inc:Int;
	public var md:Float;
	public var i:Int;
	public var x1:Int;
	public var y1:Int;
	public var ray:Bool;
	
	public function new(nx1:Int, ny1:Int, x2:Int, y2:Int, nray:Bool = false){
		x1 = nx1;
		y1 = ny1;
		lens = y2 - y1;
		lenl = x2 - x1;
		ray = nray;
		ylong = false;
		if ((lens ^ (lens >> 31)) - (lens >> 31) > (lenl ^ (lenl >> 31)) - (lenl >> 31)){
			lens ^= lenl;
			lenl ^= lens;
			lens ^= lenl;
			ylong = true;
		}
		inc = lenl < 0 ? -1 : 1;
		md = lenl == 0 ? lens : lens / lenl;
		i = 0;
	}
	
	public inline function hasNext():Bool {
		return ray || i != (lenl < 0 ? lenl - 1 : lenl + 1);
	}
	
	public inline function next():Point2D<Int> {
		var ret:Point2D<Int> = null;
		if (ylong){
			ret = new Point2D<Int>(x1 + FM.floor(i * md), y1 + i);
		} else {
			ret = new Point2D<Int>(x1 + i, y1 + FM.floor(i * md));
		}
		i += inc;
		return ret;
	}
	
	public inline function getAll(xmin:Int, ymin:Int, xmax:Int, ymax:Int):Array<Point2D<Int>> {
		if (ray) return [];
		var ret = [
			while (hasNext()){
				next();
			}
		];
		return ret;
	}
	
	public static function plot(bd:BitmapData, c:UInt, x1:Int, y1:Int, x2:Int, y2:Int):Void {
		var lens:Int = y2 - y1;
		var lenl:Int = x2 - x1;
		var ylong:Bool = false;
		if ((lens ^ (lens >> 31)) - (lens >> 31) > (lenl ^ (lenl >> 31)) - (lenl >> 31)){
			lens ^= lenl;
			lenl ^= lens;
			lens ^= lenl;
			ylong = true;
		}
		var inc:Int = lenl < 0 ? -1 : 1;
		var md:Float = lenl == 0 ? lens : lens / lenl;
		var i:Int = 0;
		if (ylong){
			while (i != lenl){
				bd.set(x1 + FM.floor(i * md), y1 + i, c);
				i += inc;
			}
		} else {
			while (i != lenl){
				bd.set(x1 + i, y1 + FM.floor(i * md), c);
				i += inc;
			}
		}
	}
}